@@ -1,3 +1,4 @@
+# pylint:disable=wrong-import-position
 from .analysis import Analysis, AnalysesHub
 from ..misc.ux import deprecated
 
@@ -23,10 +24,20 @@ from .disassembly import Disassembly
 from .variable_recovery import VariableRecovery, VariableRecoveryFast
 from .identifier import Identifier
 from .callee_cleanup_finder import CalleeCleanupFinder
-from .reaching_definitions import ReachingDefinitionAnalysis
+from .reaching_definitions import ReachingDefinitionsAnalysis
 from .calling_convention import CallingConventionAnalysis
 from .code_tagging import CodeTagging
 from .stack_pointer_tracker import StackPointerTracker
 from .dominance_frontier import DominanceFrontier
 from .decompiler import Decompiler
 from .soot_class_hierarchy import SootClassHierarchy
+from .propagator import PropagatorAnalysis
+from .xrefs import XRefsAnalysis
+from .init_finder import InitializationFinder
+from .complete_calling_conventions import CompleteCallingConventionsAnalysis
+from .typehoon import Typehoon
+from .proximity_graph import ProximityGraphAnalysis
+from .vtable import VtableFinder
+from .find_objects_static import StaticObjectFinder
+from .class_identifier import ClassIdentifier
+from .flirt import FlirtAnalysis
