@@ -1,18 +1,32 @@
-
 import logging
 from collections import defaultdict
+from typing import Union
+
+import pyvex
 
 from . import Analysis
 
+from ..utils.library import get_cpp_function_name
+from ..block import DisassemblerInsn, CapstoneInsn, SootBlockNode
+from ..codenode import BlockNode
 from .disassembly_utils import decode_instruction
-from ..block import CapstoneInsn, SootBlockNode
+
+try:
+    from ..engines import pcode
+    import pypcode
+    IRSBType = Union[pyvex.IRSB, pcode.lifter.IRSB]
+    IROpObjType = Union[pyvex.stmt.IRStmt, pypcode.PcodeOp]
+except ImportError:
+    pcode = None
+    IRSBType = pyvex.IRSB
+    IROpObjType = pyvex.stmt
 
 l = logging.getLogger(name=__name__)
 
 # pylint: disable=unidiomatic-typecheck
 
 
-class DisassemblyPiece(object):
+class DisassemblyPiece:
     addr = None
     ident = float('nan')
 
@@ -91,6 +105,33 @@ class Label(DisassemblyPiece):
         return [self.name + ':']
 
 
+class IROp(DisassemblyPiece):
+
+    __slots__ = (
+        'addr',
+        'seq',
+        'obj',
+        'irsb',
+        )
+
+    addr: int
+    seq: int
+    obj: IROpObjType
+    irsb: IRSBType
+
+    def __init__(self, addr: int, seq: int, obj: IROpObjType, irsb: IRSBType):
+        self.addr = addr
+        self.seq = seq
+        self.obj = obj
+        self.irsb = irsb
+
+    def __str__(self):
+        return str(self.obj)
+
+    def _render(self, formatting):  # pylint:disable=unused-argument
+        return [str(self)]
+
+
 class BlockStart(DisassemblyPiece):
     def __init__(self, block, parentfunc, project):
         self.addr = block.addr
@@ -139,7 +180,8 @@ class Instruction(DisassemblyPiece):
 
         self.disect_instruction()
 
-        decode_instruction(self.arch, self)
+        if isinstance(insn, CapstoneInsn):
+            decode_instruction(self.arch, self)
 
     @property
     def mnemonic(self):
@@ -234,6 +276,13 @@ class Instruction(DisassemblyPiece):
         self.opcode = Opcode(self)
         self.operands.reverse()
 
+        if not hasattr(self.insn, 'operands'):
+            # Not all disassemblers provide operands. Just use our smart split
+            for i, o in enumerate(self.operands):
+                o.reverse()
+                self.operands[i] = Operand.build(1, i, o, self)
+            return
+
         if len(self.operands) != len(self.insn.operands):
             l.error("Operand parsing failed for instruction %s. %d operands are parsed, while %d are expected.",
                     str(self.insn),
@@ -702,14 +751,29 @@ class Value(OperandPiece):
                 elif style[0] == 'label':
                     labeloffset = style[1]
                     if labeloffset == 0:
-                        return [self.project.kb.labels[self.val]]
+                        lbl = self.project.kb.labels[self.val]
+                        return [lbl]
                     return ['%s%s%#+x' % ('+' if self.render_with_sign else '', self.project.kb.labels[self.val + labeloffset], labeloffset)]
             except KeyError:
                 pass
 
         # default case
+        try:
+            func = self.project.kb.functions.get_by_addr(self.val)
+        except KeyError:
+            func = None
+
         if self.val in self.project.kb.labels:
-            return [('+' if self.render_with_sign else '') + self.project.kb.labels[self.val]]
+            lbl = self.project.kb.labels[self.val]
+            if func is not None:
+                # see if lbl == func.name and func.demangled_name != func.name. if so, we prioritize the
+                # demangled name
+                if lbl == func.name and func.name != func.demangled_name:
+                    normalized_name = get_cpp_function_name(func.demangled_name, specialized=False, qualified=True)
+                    return [normalized_name]
+            return [('+' if self.render_with_sign else '') + lbl]
+        elif func is not None:
+            return [func.demangled_name]
         else:
             if self.render_with_sign:
                 return ['%#+x' % self.val]
@@ -739,7 +803,7 @@ class FuncComment(DisassemblyPiece):
 
 
 class Disassembly(Analysis):
-    def __init__(self, function=None, ranges=None):  # pylint:disable=unused-argument
+    def __init__(self, function=None, ranges=None, include_ir=False):  # pylint:disable=unused-argument
 
         # TODO: support ranges
 
@@ -750,9 +814,11 @@ class Disassembly(Analysis):
             'labels': {},
             'instructions': {},
             'hooks': {},
+            'ir': defaultdict(list)
         }
         self.block_to_insn_addrs = defaultdict(list)
         self._func_cache = {}
+        self._include_ir = include_ir
 
         if function is not None:
             # sort them by address, put hooks before nonhooks
@@ -770,7 +836,50 @@ class Disassembly(Analysis):
             self._func_cache[f.addr] = f
             return f
 
-    def parse_block(self, block):
+    def _add_instruction_to_results(self, block: BlockNode, insn: DisassemblerInsn, bs: BlockStart) -> None:
+        """
+        Add instruction to analysis results with associated labels and comments
+        """
+        if insn.address in self.kb.labels:
+            label = Label(insn.address, self.kb.labels[insn.address])
+            self.raw_result.append(label)
+            self.raw_result_map['labels'][label.addr] = label
+        if insn.address in self.kb.comments:
+            comment = Comment(insn.address, self.kb.comments[insn.address])
+            self.raw_result.append(comment)
+            self.raw_result_map['comments'][comment.addr] = comment
+        instruction = Instruction(insn, bs)
+        self.raw_result.append(instruction)
+        self.raw_result_map['instructions'][instruction.addr] = instruction
+        self.block_to_insn_addrs[block.addr].append(insn.address)
+
+    def _add_block_ir_to_results(self, block: BlockNode, irsb: IRSBType) -> None:
+        """
+        Add lifter IR for this block
+        """
+        addr_to_ops_map = self.raw_result_map['ir']
+        addr = block.addr
+        ops = addr_to_ops_map[addr]
+
+        if irsb.statements is not None:
+            if (pcode is not None and
+                isinstance(self.project.factory.default_engine, pcode.HeavyPcodeMixin)):
+                for ins in irsb._instructions:
+                    addr = ins.address.offset
+                    addr_to_ops_map[addr].extend([
+                        IROp(addr, op.seq.uniq, op, irsb) for op in ins.ops])
+            else:
+                for seq, stmt in enumerate(irsb.statements):
+                    if isinstance(stmt, pyvex.stmt.IMark):
+                        addr = stmt.addr
+                        ops = addr_to_ops_map[addr]
+                    else:
+                        ops.append(IROp(addr, seq, stmt, irsb))
+
+    def parse_block(self, block: BlockNode) -> None:
+        """
+        Parse instructions for a given block node
+        """
         func = self.func_lookup(block)
         if func and func.addr == block.addr:
             self.raw_result.append(FuncComment(block.function))
@@ -782,8 +891,9 @@ class Disassembly(Analysis):
             hook = Hook(block.addr, bs)
             self.raw_result.append(hook)
             self.raw_result_map['hooks'][block.addr] = hook
-
         elif self.project.arch.capstone_support:
+            # Prefer Capstone first, where we are able to extract a bit more
+            # about the operands
             if block.thumb:
                 aligned_block_addr = (block.addr >> 1) << 1
                 cs = self.project.arch.capstone_thumb
@@ -796,18 +906,14 @@ class Disassembly(Analysis):
                 bytestr = block.bytestr
             self.block_to_insn_addrs[block.addr] = []
             for cs_insn in cs.disasm(bytestr, block.addr):
-                if cs_insn.address in self.kb.labels:
-                    label = Label(cs_insn.address, self.kb.labels[cs_insn.address])
-                    self.raw_result.append(label)
-                    self.raw_result_map['labels'][label.addr] = label
-                if cs_insn.address in self.kb.comments:
-                    comment = Comment(cs_insn.address, self.kb.comments[cs_insn.address])
-                    self.raw_result.append(comment)
-                    self.raw_result_map['comments'][comment.addr] = comment
-                instruction = Instruction(CapstoneInsn(cs_insn), bs)
-                self.raw_result.append(instruction)
-                self.raw_result_map['instructions'][instruction.addr] = instruction
-                self.block_to_insn_addrs[block.addr].append(cs_insn.address)
+                self._add_instruction_to_results(block, CapstoneInsn(cs_insn), bs)
+        elif pcode is not None and isinstance(self.project.factory.default_engine, pcode.HeavyPcodeMixin):
+            # When using the P-code engine, we can fall back on its disassembly
+            # in the event that Capstone does not support it
+            self.block_to_insn_addrs[block.addr] = []
+            b = self.project.factory.block(block.addr, size=block.size)
+            for insn in b.disassembly.insns:
+                self._add_instruction_to_results(block, insn, bs)
         elif type(block) is SootBlockNode:
             for raw_stmt in block.stmts:
                 stmt = SootStatement(block.addr, raw_stmt)
@@ -817,6 +923,10 @@ class Disassembly(Analysis):
         else:
             raise TypeError("")
 
+        if self._include_ir:
+            b = self.project.factory.block(block.addr, size=block.size)
+            self._add_block_ir_to_results(block, b.vex)
+
     def render(self, formatting=None):
         if formatting is None: formatting = {}
         return '\n'.join(sum((x.render(formatting) for x in self.raw_result), []))
