@@ -966,7 +966,7 @@ class SimCC:
                self.sp_delta == other.sp_delta
 
     @classmethod
-    def _match(cls, arch, args, sp_delta):
+    def _match(cls, arch, args: List, sp_delta):
         if cls.ARCH is not None and not isinstance(arch, cls.ARCH):
             return False
         if sp_delta != cls.STACKARG_SP_DIFF:
@@ -978,24 +978,32 @@ class SimCC:
         both_iter = sample_inst.both_args
         some_both_args = [next(both_iter) for _ in range(len(args))]
 
+        new_args = [ ]
         for arg in args:
             if arg not in all_fp_args and arg not in all_int_args and arg not in some_both_args:
+                if isinstance(arg, SimRegArg) and arg.reg_name in sample_inst.CALLER_SAVED_REGS:
+                    continue
                 return False
+            new_args.append(arg)
+
+        # update args (e.g., drop caller-saved register arguments)
+        args.clear()
+        args.extend(new_args)
 
         return True
 
     @staticmethod
-    def find_cc(arch, args, sp_delta):
+    def find_cc(arch: 'archinfo.Arch', args: List[SimFunctionArgument], sp_delta: int) -> Optional['SimCC']:
         """
         Pinpoint the best-fit calling convention and return the corresponding SimCC instance, or None if no fit is
         found.
 
-        :param Arch arch:       An ArchX instance. Can be obtained from archinfo.
-        :param list args:       A list of arguments.
-        :param int sp_delta:    The change of stack pointer before and after the call is made.
-        :return:                A calling convention instance, or None if none of the SimCC subclasses seems to fit the
-                                arguments provided.
-        :rtype:                 SimCC or None
+        :param arch:        An ArchX instance. Can be obtained from archinfo.
+        :param args:        A list of arguments. It may be updated by the first matched calling convention to
+                            remove non-argument arguments.
+        :param sp_delta:    The change of stack pointer before and after the call is made.
+        :return:            A calling convention instance, or None if none of the SimCC subclasses seems to fit the
+                            arguments provided.
         """
         if arch.name not in CC:
             return None
