@@ -1,8 +1,10 @@
 from typing import Dict, Optional
+import logging
 
 import claripy
-import logging
 from archinfo.arch_arm import is_arm_arch
+from angr.state_plugins.sim_action_object import _raw_ast, SimActionObject
+from angr import errors
 
 l = logging.getLogger(name=__name__)
 #l.setLevel(logging.DEBUG)
@@ -44,11 +46,16 @@ def op_concretize(op):
     if op.op == 'If':
         cases = list(claripy.reverse_ite_cases(op))
         if all(c.op == 'BVV' for _, c in cases):
-            raise CCallMultivaluedException(cases)
+            raise CCallMultivaluedException(cases, op)
     if op.op != 'BVV':
         raise SimError("Hit a symbolic conditional operation. Something has gone wildly wrong.")
     return op.args[0]
 
+def strip_simaction(val):
+    if type(val) == SimActionObject:
+        return _raw_ast(val)
+    return val
+
 class CCallMultivaluedException(Exception):
     pass
 
@@ -983,7 +990,8 @@ def amd64g_calculate_RCL(state, arg, rot_amt, eflags_in, sz):
         raise SimError('Hit a symbolic "sz" in an x86 rotate with carry instruction. Panic.')
 
     want_flags = claripy.SLT(sz, 0).is_true()
-    if want_flags: sz = -sz
+    if want_flags:
+        sz = -sz
     carry_bit_in = eflags_in[data['AMD64']['CondBitOffsets']['G_CC_SHIFT_C']]
     carry_bit_out, overflow_bit_out, arg_out = generic_rotate_with_carry(state, True, arg, rot_amt, carry_bit_in, sz)
 
@@ -1003,7 +1011,8 @@ def amd64g_calculate_RCR(state, arg, rot_amt, eflags_in, sz):
         raise SimError('Hit a symbolic "sz" in an x86 rotate with carry instruction. Panic.')
 
     want_flags = claripy.SLT(sz, 0).is_true()
-    if want_flags: sz = -sz
+    if want_flags:
+        sz = -sz
     carry_bit_in = eflags_in[data['AMD64']['CondBitOffsets']['G_CC_SHIFT_C']]
     carry_bit_out, overflow_bit_out, arg_out = generic_rotate_with_carry(state, False, arg, rot_amt, carry_bit_in, sz)
 
@@ -1432,7 +1441,8 @@ def armg_calculate_flag_n(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op == ARMG_CC_OP_MULL:
         flag = claripy.LShR(cc_dep2, 31)
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
     l.error("Unknown cc_op %s (armg_calculate_flag_n)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
 
@@ -1464,7 +1474,8 @@ def armg_calculate_flag_z(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op == ARMG_CC_OP_MULL:
         flag = arm_zerobit(state, cc_dep1 | cc_dep2)
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unknown cc_op %s (armg_calculate_flag_z)", concrete_op)
     raise SimCCallError("Unknown cc_op %s" % concrete_op)
@@ -1494,7 +1505,8 @@ def armg_calculate_flag_c(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op == ARMG_CC_OP_MULL:
         flag = (claripy.LShR(cc_dep3, 1)) & 1
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unknown cc_op %s (armg_calculate_flag_c)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
@@ -1528,7 +1540,8 @@ def armg_calculate_flag_v(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op == ARMG_CC_OP_MULL:
         flag = cc_dep3 & 1
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unknown cc_op %s (armg_calculate_flag_v)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
@@ -1590,7 +1603,8 @@ def armg_calculate_condition(state, cond_n_op, cc_dep1, cc_dep2, cc_dep3):
         zf = armg_calculate_flag_z(state, cc_op, cc_dep1, cc_dep2, cc_dep3)
         flag = inv ^ (1 & ~(zf | (nf ^ vf)))
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unrecognized condition %d in armg_calculate_condition", concrete_cond)
     raise SimCCallError("Unrecognized condition %d in armg_calculate_condition" % concrete_cond)
@@ -1753,7 +1767,8 @@ def arm64g_calculate_flag_c(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op in (ARM64G_CC_OP_LOGIC32, ARM64G_CC_OP_LOGIC64):
         flag = claripy.BVV(0, 64) # C after logic is zero on arm64
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unknown cc_op %s (arm64g_calculate_flag_c)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
@@ -1809,7 +1824,8 @@ def arm64g_calculate_flag_v(state, cc_op, cc_dep1, cc_dep2, cc_dep3):
     elif concrete_op in (ARM64G_CC_OP_LOGIC32, ARM64G_CC_OP_LOGIC64):
         flag = claripy.BVV(0, 64)
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unknown cc_op %s (arm64g_calculate_flag_v)", cc_op)
     raise SimCCallError("Unknown cc_op %s" % cc_op)
@@ -1865,7 +1881,8 @@ def arm64g_calculate_condition(state, cond_n_op, cc_dep1, cc_dep2, cc_dep3):
         zf = arm64g_calculate_flag_z(state, cc_op, cc_dep1, cc_dep2, cc_dep3)
         flag = inv ^ (1 & ~(zf | (nf ^ vf)))
 
-    if flag is not None: return flag
+    if flag is not None:
+        return flag
 
     l.error("Unrecognized condition %d in arm64g_calculate_condition", concrete_cond)
     raise SimCCallError("Unrecognized condition %d in arm64g_calculate_condition" % concrete_cond)
@@ -1875,6 +1892,10 @@ def arm64g_calculate_condition(state, cond_n_op, cc_dep1, cc_dep2, cc_dep3):
 #
 
 def _get_flags(state) -> claripy.ast.bv.BV:
+    cc_op = strip_simaction(state.regs.cc_op)
+    cc_dep1 = strip_simaction(state.regs.cc_dep1)
+    cc_dep2 = strip_simaction(state.regs.cc_dep2)
+    cc_ndep = strip_simaction(state.regs.cc_ndep)
     if state.arch.name == 'X86':
         func = x86g_calculate_eflags_all
     elif state.arch.name == 'AMD64':
@@ -1887,10 +1908,17 @@ def _get_flags(state) -> claripy.ast.bv.BV:
         l.warning("No such thing as a flags register for arch %s", state.arch.name)
         return None
     try:
-        return func(state, state.regs.cc_op, state.regs.cc_dep1, state.regs.cc_dep2, state.regs.cc_ndep)
+        return func(state, cc_op, cc_dep1, cc_dep2, cc_ndep)
     except CCallMultivaluedException as e:
-        cases = e.args[0]
-        return claripy.ite_cases([(case, func(state, value, state.regs.cc_dep1, state.regs.cc_dep2, state.regs.cc_ndep)) for case, value in cases], 0)
+        cases, to_replace = e.args
+        args = [cc_op, cc_dep1, cc_dep2, cc_ndep]
+        for i, arg in enumerate(args):
+            if arg is to_replace:
+                break
+        else:
+            raise errors.UnsupportedCCallError("Trying to concretize a value which is not an argument")
+        return claripy.ite_cases([(case, func(state, *args[:i], value_, *args[i + 1:])) for case, value_ in cases], 0)
+
 
 def _concat_flags(nbits, flags_vec):
     """
