@@ -200,7 +200,11 @@ class Tracer(ExplorationTechnique):
         """
         if self._aslr:
             # if we don't know whether there is any slide, we need to identify the slides via heuristics
-            for obj in self.project.loader.all_elf_objects:
+            for obj in self.project.loader.all_objects:
+                # do not analyze pseudo-objects
+                if obj.binary_basename.startswith('cle##'):
+                    continue
+
                 # heuristic 1: non-PIC  objects are loaded without aslr slides
                 if not obj.pic:
                     self._aslr_slides[obj] = 0
@@ -233,7 +237,10 @@ class Tracer(ExplorationTechnique):
                     raise AngrTracerError("Trace seems ambiguous with respect to what the ASLR slides are for %s. This is surmountable, please open an issue." % obj)
         else:
             # if we know there is no slides, just trust the address in the loader
-            for obj in self.project.loader.all_elf_objects:
+            for obj in self.project.loader.all_objects:
+                # do not analyze pseudo-objects
+                if obj.binary_basename.startswith('cle##'):
+                    continue
                 self._aslr_slides[obj] = 0
             self._current_slide = 0
 
@@ -525,8 +532,8 @@ class Tracer(ExplorationTechnique):
             # termination! will be handled by filter
             pass
         elif self.project.is_hooked(state.addr) and not self.project.loader.extern_object.contains_addr(state.addr):
-           # handle simprocedures
-           self._sync_return(state, idx)
+            # handle simprocedures
+            self._sync_return(state, idx)
         elif self._compare_addr(self._trace[idx + 1], state.addr):
             # normal case
             state.globals['trace_idx'] = idx + 1
