@@ -162,12 +162,14 @@ struct register_value_t {
 	}
 };
 
-template <>
-struct std::hash<register_value_t> {
-	std::size_t operator()(const register_value_t &value) const {
-		return value.operator()(value);
-	}
-};
+namespace std {
+	template <>
+	struct hash<register_value_t> {
+		std::size_t operator()(const register_value_t &value) const {
+			return value.operator()(value);
+		}
+	};
+}
 
 struct instr_details_t {
 	address_t instr_addr;
@@ -303,10 +305,11 @@ enum stop_t {
 	STOP_UNSUPPORTED_STMT_UNKNOWN,
 	STOP_UNSUPPORTED_EXPR_GETI,
 	STOP_UNSUPPORTED_EXPR_UNKNOWN,
-	STOP_UNKNOWN_MEMORY_WRITE,
+	STOP_UNKNOWN_MEMORY_WRITE_SIZE,
 	STOP_SYMBOLIC_MEM_DEP_NOT_LIVE,
 	STOP_SYSCALL_ARM,
 	STOP_SYMBOLIC_MEM_DEP_NOT_LIVE_CURR_BLOCK,
+	STOP_X86_CPUID,
 };
 
 typedef std::vector<std::pair<taint_entity_t, std::unordered_set<taint_entity_t>>> taint_vector_t;
@@ -332,13 +335,15 @@ struct instruction_taint_entry_t {
 	uint32_t mem_read_size;
 
 	bool has_memory_read;
-	bool has_memory_write;
+
+	// Count number of bytes written to memory by the instruction
+	uint32_t mem_write_size;
 
 	bool operator==(const instruction_taint_entry_t &other_instr_deps) const {
 		return (taint_sink_src_map == other_instr_deps.taint_sink_src_map) &&
 			   (dependencies == other_instr_deps.dependencies) &&
 			   (has_memory_read == other_instr_deps.has_memory_read) &&
-			   (has_memory_write == other_instr_deps.has_memory_write);
+			   (mem_write_size == other_instr_deps.mem_write_size);
 	}
 
 	void reset() {
@@ -350,8 +355,8 @@ struct instruction_taint_entry_t {
 		ite_cond_entity_list.clear();
 		taint_sink_src_map.clear();
 		has_memory_read = false;
-		has_memory_write = false;
 		mem_read_size = 0;
+		mem_write_size = 0;
 		unmodified_dep_regs.clear();
 		return;
 	}
@@ -363,13 +368,24 @@ struct block_taint_entry_t {
 	// Track instruction that sets a VEX temp and list of VEX temps on which its value depends on
 	std::unordered_map<taint_entity_t, std::pair<address_t, std::unordered_set<taint_entity_t>>> vex_temp_deps;
 	address_t exit_stmt_instr_addr;
+	bool has_cpuid_instr;
 	bool has_unsupported_stmt_or_expr_type;
 	stop_t unsupported_stmt_stop_reason;
 	std::unordered_set<taint_entity_t> block_next_entities;
 
+	block_taint_entry_t() {
+		block_instrs_taint_data_map.clear();
+		exit_stmt_guard_expr_deps.clear();
+		exit_stmt_instr_addr = 0;
+		vex_temp_deps.clear();
+		has_cpuid_instr = false;
+		has_unsupported_stmt_or_expr_type = false;
+		block_next_entities.clear();
+	}
+
 	bool operator==(const block_taint_entry_t &other_entry) const {
 		return (block_instrs_taint_data_map == other_entry.block_instrs_taint_data_map) &&
-			   (vex_temp_deps == other_entry.vex_temp_deps) &&
+			   (vex_temp_deps == other_entry.vex_temp_deps) && (has_cpuid_instr == other_entry.has_cpuid_instr) &&
 			   (exit_stmt_instr_addr == other_entry.exit_stmt_instr_addr) &&
 			   (exit_stmt_guard_expr_deps == other_entry.exit_stmt_guard_expr_deps) &&
 			   (block_next_entities == other_entry.block_next_entities);
@@ -432,6 +448,18 @@ struct mem_write_t {
 	std::vector<taint_t> previous_taint;
 };
 
+struct mem_write_taint_t {
+	address_t instr_addr;
+	bool is_symbolic;
+	uint32_t size;
+
+	mem_write_taint_t(address_t write_instr, bool symbolic, uint32_t write_size) {
+		instr_addr = write_instr;
+		is_symbolic = symbolic;
+		size = write_size;
+	}
+};
+
 struct mem_update_t {
 	address_t address;
 	uint64_t length;
@@ -464,7 +492,7 @@ class State {
 	// Memory write instruction address -> is_symbolic
 	// TODO: Need to modify memory write taint handling for architectures that perform multiple
 	// memory writes in a single instruction
-	std::unordered_map<address_t, bool> mem_writes_taint_map;
+	std::vector<mem_write_taint_t> block_mem_writes_taint_data;
 
 	// List of instructions in a block that should be executed symbolically. These are stored
 	// separately for easy rollback in case of errors.
@@ -532,6 +560,7 @@ class State {
 	bool is_symbolic_register(vex_reg_offset_t reg_offset, int64_t reg_size) const;
 	bool is_symbolic_temp(vex_tmp_id_t temp_id) const;
 
+	bool is_cpuid_in_block(address_t block_address, int32_t block_size);
 	VEXLiftResult* lift_block(address_t block_address, int32_t block_size);
 
 	void mark_register_symbolic(vex_reg_offset_t reg_offset, int64_t reg_size);
